//
//  Helpers.js
//  Kite
//
//  Created by Joe Rhodes
//  Copyright (c) 2015 Joe Rhodes. All rights reserved.
//


HelpersObj = function(fileURL, sketchDoc) {

    this.kDateFormatter = null
    this.kSessionUUID = NSUUID.UUID().UUIDString()
}

HelpersObj.prototype = Object.create(Object.prototype, {

    logError: {
        value: function(error) {
            log("Error Caught: " + error)
            log("Line: " + error.line + " Stack: " + error.stack + " Column: " + error.column + " SoruceURL: " + error.sourceURL)
        }
    },

    showDialogWithTitleMessage: {
        value: function(title, message) {
            [[NSApplication sharedApplication] displayDialog:message withTitle:title]
        }
    },

    stringFromNSDate: {
        value: function(nsdate) {

            if (this.kDateFormatter == null) {
                this.kDateFormatter = [[NSDateFormatter alloc] init]
                this.kDateFormatter.dateStyle = NSDateFormatterMediumStyle;
                this.kDateFormatter.timeStyle = NSDateFormatterMediumStyle;
            }

            return this.kDateFormatter.stringFromDate(nsdate)
        }
    },

    sketchLikeColorWithRGBA: {
        value: function(r, g, b, a) {
            return {
                red: function() { return r },
                green: function() { return g },
                blue: function() { return b },
                alpha: function() { return a },
            }
        }
    },

    archiveFromSketchColor: {
        value: function(sketchColor) {
            return {
                components: {
                    r: sketchColor.red(),
                    g: sketchColor.green(),
                    b: sketchColor.blue(),
                    a: sketchColor.alpha()
                }
            }
        }
    },

    msRectFromCGRect: {
        value: function(cgRect) {
            return {
                x: function() { return cgRect.origin.x },
                y: function() { return cgRect.origin.y },
                width: function() { return cgRect.size.width },
                height: function() { return cgRect.size.height },
            }
        }
    },

    influenceRectForSketchLayer: {
        value: function(sketchLayer) {

            var influenceRect = sketchLayer.frame()

            if (sketchLayer.influenceRectForFrame) {
                influenceRect = this.sketchLayer.influenceRectForFrame()
            }

            else if (sketchLayer.immutableModelObject &&
                     sketchLayer.immutableModelObject().influenceRectForFrameInDocument)
            {
                influenceRect = sketchLayer.immutableModelObject().influenceRectForFrameInDocument(nil);
            }

            else if (sketchLayer.influenceRectForBounds)
            {
                var originalRotation = sketchLayer.rotation()
                sketchLayer.setRotation(0)

                var influenceRectForBounds = sketchLayer.influenceRectForBounds()
                var frame = sketchLayer.frame()
                influenceRectForBounds.origin.x += frame.x()
                influenceRectForBounds.origin.y += frame.y()
                sketchLayer.setRotation(originalRotation)

                influenceRect = influenceRectForBounds
            }

            return this.msRectFromCGRect(influenceRect)
        }
    },

    stringFromSize: {
        value: function(sizeObj) {
            return "{" + sizeObj.width + ", " + sizeObj.height + "}"
        }
    },

    stringFromPoint: {
        value: function(pointObj) {
            return "{" + pointObj.x + ", " + pointObj.y + "}"
        }
    },

    stringFromRect: {
        value: function(rectObj) {
            return "{{" + rectObj.origin.x + ", " + rectObj.origin.y + "}, " + "{" + rectObj.size.width + ", " + rectObj.size.height + "}}"
        }
    },

    writeArchiveToDestination: {
        value: function(archivedRep, destinationURL) {

            var jsonWriteError = null
            var archivedData = [NSJSONSerialization dataWithJSONObject:archivedRep options: NSJSONWritingPrettyPrinted error:jsonWriteError]

            if (archivedData != null)
            {
                var writeWasSuccessful = [archivedData writeToURL:destinationURL atomically:true]

                if (writeWasSuccessful == false) {
                    log("Error writing DocumentInfo data to file: " + destinationURL.path())
                }

                return writeWasSuccessful
            }
            else {
                log("Error writing DocumentInfo JSON! " + jsonWriteError)
                return false
            }
        }
    },

    tempURL: {
        value: function() {
            var path = NSString.stringWithString("KiteSketchExport")
            var tempPath = NSTemporaryDirectory().stringByAppendingPathComponent(path)

            return [NSURL fileURLWithPath:tempPath]
        }
    },

    tempSVGExportURL: {
        value: function() {

            var sessionUUID = this.kSessionUUID
            var subfolderName = NSString.stringWithString("SVGImageExport-").stringByAppendingString(sessionUUID)

            return this.tempURL().URLByAppendingPathComponent(subfolderName)
        }
    },

    // Dump Object, taken from github.com/sketchplugins/plugin-requests
    //
    dumpObject: {
        value: function dump(obj) {
          log("######################################")
          log("## Dumping object " + obj )
          log("## obj class is: " + [obj className])
          log("######################################")

          log("obj.properties:")
          log([obj class].mocha().properties())
          log("obj.propertiesWithAncestors:")
          log([obj class].mocha().propertiesWithAncestors())

          log("obj.classMethods:")
          log([obj class].mocha().classMethods())
          log("obj.classMethodsWithAncestors:")
          log([obj class].mocha().classMethodsWithAncestors())

          log("obj.instanceMethods:")
          log([obj class].mocha().instanceMethods())
          log("obj.instanceMethodsWithAncestors:")
          log([obj class].mocha().instanceMethodsWithAncestors())

          log("obj.protocols:")
          log([obj class].mocha().protocols())
          log("obj.protocolsWithAncestors:")
          log([obj class].mocha().protocolsWithAncestors())

          log("obj.treeAsDictionary():")
          log(obj.treeAsDictionary())
        }
    },
})

var Helpers = new HelpersObj()
